﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Data;
using System.Data.OleDb;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace ProFormAPICSharp
{
	public partial class Form1 : Form
	{
        private bool m_bFileIsOpen;
        private int m_nSymTabVer;

		[DllImport("ProFormAPI.dll")]
		private static extern int SPLoadSPPrompts(string pathToSPPromptsFile);

		[DllImport("ProFormAPI.dll")]
		private static extern int SPLoadUserPrompts(string pathToUserPromptsFile);
		
		[DllImport("ProFormAPI.dll")]
        private static extern int CreateProFormFile(string UserID, string Template, string Prefix, string Suffix, int TabVersion, string FileNumCode, StringBuilder NewFileID);

		[DllImport("ProFormAPI.dll")]
		private static extern int SPOpenFile(string UserID, string FileID);

		[DllImport("ProFormAPI.dll")]
		private static extern int SPOpenFileRO(string FileID);

        [DllImport("ProFormAPI.dll")]
        private static extern int SPSaveFile();

        [DllImport("ProFormAPI.dll")]
        private static extern int SPCloseFile();

		[DllImport("ProFormAPI.dll")]
		private static extern int SPGetFieldValue(string FieldName, StringBuilder ReturnValue, int MaxSize);

        [DllImport("ProFormAPI.dll")]
        private static extern int SPSetFieldValue(string FieldName, string value);

        [DllImport("kernel32.dll")]
        private static extern bool FreeLibrary([In]IntPtr hModule);

		public Form1()
		{
			InitializeComponent();

            tbUserID.Text = "Default";
            tbFileNumID.Text = "Default";
            m_bFileIsOpen = false;
            m_nSymTabVer = 1;

			// Initialize SoftPro prompts (pass in path to spprmpts.txt)
			int retval = SPLoadSPPrompts(@"D:\shared\ENT\Docs");

			// Initialize SoftPro prompts (pass in path to Usrprmpts.txt
			retval = SPLoadUserPrompts(@"D:\shared\ENT\Docs");
		}

        private void btnDone_Click(object sender, EventArgs e)
        {
            UnloadImportedDll("ProFormAPI.dll");
            this.Close();
        }

        private void btnCreateFile_Click(object sender, EventArgs e)
        {
            string userID = tbUserID.Text;
            string template = tbTemplate.Text;
            string prefix = tbPrefix.Text;
            string suffix = tbSuffix.Text;
            string fileNumCode = tbFileNumID.Text;
            StringBuilder newFileID = new StringBuilder();
            int retCode = 0;

            retCode = CreateProFormFile(userID, template, prefix, suffix, m_nSymTabVer, fileNumCode, newFileID);
            if (retCode >= 0)
            {
                tbFileID.Text = newFileID.ToString();
            }
            DisplayError(retCode);
        }

        private void btnOpenFile_Click(object sender, EventArgs e)
        {
            string fileID = tbFileID.Text;
            string userID = tbUserID.Text;
            int retCode = 0;

            retCode = SPOpenFile(userID, fileID);
            tbStatus.Text = (retCode == 0) ? "Open" : "Closed";
            m_bFileIsOpen = (retCode == 0);
            DisplayError(retCode);
        }

        private void btnSaveFile_Click(object sender, EventArgs e)
        {
            int retCode = 0;

            if (m_bFileIsOpen)
            {
                retCode = SPSaveFile();
                DisplayError(retCode);
            }
        }

        private void btnCloseFile_Click(object sender, EventArgs e)
        {
            int retCode = 0;

            if (m_bFileIsOpen)
            {
                retCode = SPCloseFile();
                tbStatus.Text = (retCode == 0) ? "Closed" : "Left open";
                m_bFileIsOpen = (retCode != 0);
                DisplayError(retCode);
            }
        }

        private void btnGetFieldValue_Click(object sender, EventArgs e)
        {
            string fieldName = tbFieldCode.Text;
            StringBuilder fieldValue = new StringBuilder(255);
            int retCode = 0;

            retCode = SPGetFieldValue(fieldName, fieldValue, 255);
            if (retCode > 0)
            {
                tbFieldValue.Text = fieldValue.ToString(0, retCode);
            }
            else
            {
                DisplayError(retCode);
            }
        }

        private void btnSetFieldValue_Click(object sender, EventArgs e)
        {
            string fieldName = tbFieldCode.Text;
            string fieldValue = tbFieldValue.Text;
            int retCode = 0;

            retCode = SPSetFieldValue(fieldName, fieldValue);
            DisplayError(retCode);
        }

        private void DisplayError(int errorCode)
        {
            switch (errorCode)
            {
                case 0:
                    tbMessage.Text = "Success";
                    break;
                case -1:
                    tbMessage.Text = "-1: driver dlg error: File not open for processing";
                    break;
                case -101:
                    tbMessage.Text = "-101: Internal application error";
                    break;
                case -102:
                    tbMessage.Text = "-102: File open error";
                    break;
                case -103:
                    tbMessage.Text = "-103: FileID not found in Search table";
                    break;
                case -104:
                    tbMessage.Text = "-104: Field name not found in file";
                    break;
                case -105:
                    tbMessage.Text = "-105: Field value not valid to field type";
                    break;
                case -106:
                    tbMessage.Text = "-106: FileID was not found in the Search table";
                    break;
                case -107:
                    tbMessage.Text = "-107: Search table access error encountered";
                    break;
                case -108:
                    tbMessage.Text = "-108: ProForm File already locked";
                    break;
                case -109:
                    tbMessage.Text = "-109: Error during save of file";
                    break;
                case -110:
                    tbMessage.Text = "-110: Error retrieving next available file number";
                    break;
                case -111:
                    tbMessage.Text = "-111: Prefix required (setting in SPAdmin)";
                    break;
                case -112:
                    tbMessage.Text = "-112: Prefix not found in list of available values (SPAdmin)";
                    break;
                case -113:
                    tbMessage.Text = "-113: Suffix required (SPAdmin)";
                    break;
                case -114:
                    tbMessage.Text = "-114: Suffix not found in list of available values (SPAdmin)";
                    break;
                case -115:
                    tbMessage.Text = "-115: Error encountered during creation of new file";
                    break;
                case -116:
                    tbMessage.Text = "-116: Overlay file not found or is invalid";
                    break;
                case -117:
                    tbMessage.Text = "-117: Unable to update the file numbering manager with next value";
                    break;
                case -118:
                    tbMessage.Text = "-118: Error retrieving the field data";
                    break;
                case -119:
                    tbMessage.Text = "-119: File being created already exists in the data directory";
                    break;
                case -120:
                    tbMessage.Text = "-120: File exception encountered during save of file";
                    break;
                case -121:
                    tbMessage.Text = "-121: Error trying to connect to the ProForm Database ";
                    break;
                case -122:
                    tbMessage.Text = "-122: File Numbering Manager Code was not sent in";
                    break;
                case -123:
                    tbMessage.Text = "-123: User name not found in ProForm Preferences table";
                    break;
                case -124:
                    tbMessage.Text = "-124: Failed to open Field Extraction list file";
                    break;
                default:
                    tbMessage.Text = errorCode.ToString() + " = Unhandled error code encountered";
                    break;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            string fileID = tbFileID.Text;
            int retCode = 0;

            retCode = SPOpenFileRO(fileID);
            tbStatus.Text = (retCode == 0) ? "Open" : "Closed";
            m_bFileIsOpen = (retCode == 0);
            DisplayError(retCode);
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            m_nSymTabVer = comboBox1.SelectedIndex;
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {

        }

        public static void UnloadImportedDll(string DllPath)
        {
            foreach (System.Diagnostics.ProcessModule mod in System.Diagnostics.Process.GetCurrentProcess().Modules)
            {
                if (mod.FileName.Contains(DllPath))
                {
                    FreeLibrary(mod.BaseAddress);
                }
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            string mdbPath = "C:\\Sandbox\\STD20\\pfpx.mdb";
            string cmdString = "SELECT FirmFile FROM Entities WHERE FirmFile LIKE '#%'";
            int count = 0;
            StringBuilder itemStr = new StringBuilder();
            string JETConnectionString = "Provider=Microsoft.Jet.OLEDB.4.0;" +
                                         "Data Source=" + mdbPath;

            OleDbConnection JETConnection = new OleDbConnection(JETConnectionString);
            JETConnection.Open();

            // create query cmd
            DataTable dataTable = new DataTable();
            OleDbCommand command = new OleDbCommand(cmdString, JETConnection);
            OleDbDataAdapter adapter = new OleDbDataAdapter(command);
            adapter.FillSchema(dataTable, SchemaType.Source);
            adapter.Fill(dataTable);

            count = dataTable.Rows.Count;

            JETConnection.Close();
        }
	}
}
